# -*- coding: utf-8 -*-

"""
httppost for Isaac

Copyright (C) 2004 - Marco Andreini, Cristian Lucchesi

GPL 2.1

"""

import httplib
import re
from urlparse import urlparse
from random import randrange


class Boundary:
    """boundary mime object"""

    __start = None

    def __init__(self):
        self.__data = []

    def start(self):
        """add a start boundary"""

        self.__data.append(self.__start)

    def add(self, item):
        """add an item to mime object"""

        assert not item is self.__start
        self.__data.append(item)

    def makeBoundary(self):
        """make a random boundary not present in data"""

        # NOTE: RFC tell max is 70 chars.
        maxwidth = 30 # 36 - 6
        width = 4
        found = False
        while not found:
            fmt = "-----%%0%dd-" % (width, )
            boundary = fmt % (randrange(10 ** width), )
            cre = re.compile("^--" + re.escape(boundary) + "(--)?$",
                             re.MULTILINE)
            found = True
            for item in self.__data:
                if not item is self.__start and cre.search(item):
                    found = False
                    break
            if width < maxwidth:
                width += 1
        return boundary

    def end(self):
        """add an end boundary and return (boundary, mime-content)"""

        boundary = self.makeBoundary()
        startBoundary = "--" + boundary
        endBoundary = "--" + boundary + "--"
        return boundary, '\r\n'.join([item == self.__start and startBoundary or
                                      item for item in self.__data] + [endBoundary])


class HTTPPost:
    """http post

    (interface like httpsession.py)
    """

    def __init__(self, url):
        self.url = url
        self.schema, self.server, self.path, self.params, \
                     self.query, self.fragment = urlparse(url)
        if self.schema == "http":
            self.connection = httplib.HTTPConnection(self.server)
        elif self.schema == "https":
            self.connection = httplib.HTTPSConnection(self.server)
        else:
            raise HTTPPostError, "unsupported uri schema %s" % (self.schema, )
        self.headers = []
        self.postFiles = []
        self.postParams = []
        
    def addHeader(self, key, value):
        """add an header key: value"""

        self.headers.append((key, value))

    def addCookie(self, value):
        """add cookie to headers"""

        self.addHeader("Cookie", value)

    def addFile(self, name, filename, contentType, value):
        """add a file to post"""

        self.postFiles.append((name, filename, contentType, value))

    def addParam(self, name, value):
        """add a param to post"""

        self.postParams.append((name, value))

    def getreply(self):
        """transmit headers + data to server"""

        boundary = Boundary()
        for key, value in self.postParams:
            boundary.start()
            boundary.add("Content-Disposition:  form-data; name=\"%s\"" % (key, ))
            if not value is None:
                boundary.add(value)

        for name, filename, contentType, value in self.postFiles:
            boundary.start()
            boundary.add("Content-Disposition: form-data; name=\"%s\"; filename=\"%s\"" \
                        % (name, filename))
            boundary.add("Content-Type: %s" % (contentType, ))
            boundary.add(value)
        boundaryStr, body = boundary.end()
        self.addHeader("Content-Type", "multipart/form-data; boundary=\"%s\"" % \
                       (boundaryStr, ))
        self.connection.request("POST", self.url, body, dict(self.headers))

        response = self.connection.getresponse()
        # Note: is possible to reuse connection.
        self.connection.close()
        # FIXME: header {} ?
        return response.status, response.msg, {}

    
if __name__ == "__main__":
    pass
